'use strict';

const request = require('request');
const util = require('util');

const REQUEST_TIMEOUT = 10000;
const MSG_ATTRIBUTE_KEYS = ["Name", "Type", "Value"];
const UNKNOWN_RESOURCE = "unknown";

exports.handler = (event, context, callback) => {
   let events = convertRecordsToAnsEvents(event.Records);
	
    let hasError = false;
    let tempError = undefined;

    for (let event of events) {
        console.log("Posting event: ");
        console.log(util.inspect(event, {depth: null}));
        let options = buildOptions(event);
        request(options, function(error, response, body) {
            if(error) {
                hasError = true;
                tempError = error;
                return;
            }
            console.log("Successfully posted to ANS");
        });
    }

    return hasError ? callback(null, 202) : callback(Error(tempError));
};

const convertRecordsToAnsEvents = (records) => {
    let tempAnsEvents = [];

    for(let record of records) {
        tempAnsEvents.push(convertToAnsPayload(record));
    }

    return tempAnsEvents;
};

const buildOptions = (body) => {
    return {
        url: process.env.ANS_SERVICE_API,
        method: "POST",
        auth: {
            user: process.env.CLIENT_ID,
            password: process.env.CLIENT_SECRET
        },
        json: body,
        timeout: REQUEST_TIMEOUT
    };
};

const convertToAnsPayload = (notification) => {
    let snsNotification = notification || {};

    return {
        "eventType": snsNotification.Sns.Type,
        "severity": "INFO",
        "category": "NOTIFICATION",
        "subject": snsNotification.Sns.Subject,
        "body": snsNotification.Sns.Message,
        "resource": {
            "resourceName": UNKNOWN_RESOURCE,
            "resourceType": UNKNOWN_RESOURCE
        },
        "tags": buildTags(snsNotification)
    }
}

const buildTags = (snsNotification) => {
    let tempTags = {
        "sns:messageId": snsNotification.Sns.MessageId,
        "sns:topicArn": snsNotification.Sns.TopicArn,
        "sns:timestamp": snsNotification.Sns.Timestamp,
        "sns:eventSource": snsNotification.EventSource,
        "sns:subscriptionArn": snsNotification.EventSubscriptionArn
    };
    let messageAttributes = extractMessageAttributes(snsNotification.Sns.MessageAttributes);

    return Object.assign(tempTags, messageAttributes);
};

const extractMessageAttributes = (attributes) => {
    let tempAttrubutes = {};

    let buildKey = function(parentKey, childKey) {
        return `${parentKey}.${childKey}`;
    };

    for (let [parentKey, value] of Object.entries(attributes)) {
        for (let messageAttributeKey of MSG_ATTRIBUTE_KEYS) {
            if (value.hasOwnProperty(messageAttributeKey)) {
                let key = buildKey(parentKey, messageAttributeKey);
                tempAttrubutes[`${key}`] = value[messageAttributeKey];
            }
        }
    }
    
    return tempAttrubutes;
};